@@ -6,10 +6,9 @@ use base 'Class::Accessor::Grouped';
 use mro 'c3';
 use Carp::Clan qw/^DBIx::Class/;
 use Scalar::Util 'weaken';
-use DBIx::Class::Schema::Loader::Utils qw/split_name slurp_file array_eq/;
+use DBIx::Class::Schema::Loader::Utils qw/split_name slurp_file array_eq apply uniq/;
 use Try::Tiny;
-use List::Util 'first';
-use List::MoreUtils qw/apply uniq any/;
+use List::Util qw/all any first/;
 use namespace::clean;
 use Lingua::EN::Inflect::Phrase ();
 use Lingua::EN::Tagger ();
@@ -18,12 +17,12 @@ use String::ToIdentifier::EN::Unicode ();
 use Class::Unload ();
 use Class::Inspector ();
 
-our $VERSION = '0.07015';
+our $VERSION = '0.07049';
 
 # Glossary:
 #
-# remote_relname -- name of relationship from the local table referring to the remote table
-# local_relname  -- name of relationship from the remote table referring to the local table
+# local_relname  -- name of relationship from the local table referring to the remote table
+# remote_relname -- name of relationship from the remote table referring to the local table
 # remote_method  -- relationship type from remote table to local table, usually has_many
 
 =head1 NAME
@@ -47,8 +46,8 @@ Arguments: $loader object
 
 =head2 generate_code
 
-Arguments: 
-    
+Arguments:
+
     [
         [ local_moniker1 (scalar), fk_info1 (arrayref), uniq_info1 (arrayref) ]
         [ local_moniker2 (scalar), fk_info2 (arrayref), uniq_info2 (arrayref) ]
@@ -95,16 +94,18 @@ This method will return the generated relationships as a hashref keyed on the
 class names.  The values are arrayrefs of hashes containing method name and
 arguments, like so:
 
-  {
-      'Some::Source::Class' => [
-          { method => 'belongs_to', arguments => [ 'col1', 'Another::Source::Class' ],
-          { method => 'has_many', arguments => [ 'anothers', 'Yet::Another::Source::Class', 'col15' ],
-      ],
-      'Another::Source::Class' => [
-          # ...
-      ],
-      # ...
-  }
+    {
+        'Some::Source::Class' => [
+            { method => 'belongs_to', arguments => [
+              'col1', 'Another::Source::Class' ] },
+            { method => 'has_many', arguments => [
+              'anothers', 'Yet::Another::Source::Class', 'col15' ] },
+        ],
+        'Another::Source::Class' => [
+            # ...
+        ],
+        # ...
+    }
 
 =cut
 
@@ -116,6 +117,7 @@ __PACKAGE__->mk_group_accessors('simple', qw/
     relationship_attrs
     rel_collision_map
     rel_name_map
+    allow_extra_m2m_cols
     _temp_classes
     __tagger
 /);
@@ -134,12 +136,11 @@ sub new {
 
     my $self = {
         loader             => $loader,
-        schema             => $loader->schema,
-        inflect_plural     => $loader->inflect_plural,
-        inflect_singular   => $loader->inflect_singular,
-        relationship_attrs => $loader->relationship_attrs,
-        rel_collision_map  => $loader->rel_collision_map,
-        rel_name_map       => $loader->rel_name_map,
+        (map { $_ => $loader->$_ } qw(
+            schema inflect_plural inflect_singular
+            relationship_attrs rel_collision_map
+            rel_name_map allow_extra_m2m_cols
+        )),
         _temp_classes      => [],
     };
 
@@ -149,8 +150,8 @@ sub new {
 
     # validate the relationship_attrs arg
     if( defined $self->relationship_attrs ) {
-        ref $self->relationship_attrs eq 'HASH'
-            or croak "relationship_attrs must be a hashref";
+        (ref $self->relationship_attrs eq 'HASH' || ref $self->relationship_attrs eq 'CODE')
+            or croak "relationship_attrs must be a hashref or coderef";
     }
 
     return $self;
@@ -249,24 +250,39 @@ sub _default_relationship_attrs { +{
     },
 } }
 
-# accessor for options to be passed to each generated relationship
-# type.  take single argument, the relationship type name, and returns
-# either a hashref (if some options are set), or nothing
+# Accessor for options to be passed to each generated relationship type. takes
+# the relationship type name and optionally any attributes from the database
+# (such as FK ON DELETE/UPDATE and DEFERRABLE clauses), and returns a
+# hashref or undef if nothing is set.
+#
+# The attributes from the database override the default attributes, which in
+# turn are overridden by user supplied attributes.
 sub _relationship_attrs {
-    my ( $self, $reltype ) = @_;
+    my ( $self, $reltype, $db_attrs, $params ) = @_;
     my $r = $self->relationship_attrs;
 
     my %composite = (
         %{ $self->_default_relationship_attrs->{$reltype} || {} },
-        %{ $r->{all} || {} }
+        %{ $db_attrs || {} },
+        (
+            ref $r eq 'HASH' ? (
+                %{ $r->{all} || {} },
+                %{ $r->{$reltype} || {} },
+            )
+            :
+            ()
+        ),
     );
 
-    if( my $specific = $r->{$reltype} ) {
-        while( my ($k,$v) = each %$specific ) {
-            $composite{$k} = $v;
-        }
+    if (ref $r eq 'CODE') {
+        $params->{attrs} = \%composite;
+
+        my %ret = %{ $r->(%$params) || {} };
+
+        %composite = %ret if %ret;
     }
-    return \%composite;
+
+    return %composite ? \%composite : undef;
 }
 
 sub _strip_id_postfix {
@@ -278,10 +294,10 @@ sub _strip_id_postfix {
 }
 
 sub _remote_attrs {
-    my ($self, $local_moniker, $local_cols) = @_;
+    my ($self, $local_moniker, $local_cols, $fk_attrs, $params) = @_;
 
-    # get our base set of attrs from _relationship_attrs, if present
-    my $attrs = $self->_relationship_attrs('belongs_to') || {};
+    # get our set of attrs from _relationship_attrs, which uses the FK attrs if available
+    my $attrs = $self->_relationship_attrs('belongs_to', $fk_attrs, $params) || {};
 
     # If any referring column is nullable, make 'belongs_to' an
     # outer join, unless explicitly set by relationship_attrs
@@ -311,22 +327,22 @@ sub _normalize_name {
     return join '_', map lc, @words;
 }
 
-sub _remote_relname {
+sub _local_relname {
     my ($self, $remote_table, $cond) = @_;
 
-    my $remote_relname;
+    my $local_relname;
     # for single-column case, set the remote relname to the column
     # name, to make filter accessors work, but strip trailing _id
     if(scalar keys %{$cond} == 1) {
         my ($col) = values %{$cond};
         $col = $self->_strip_id_postfix($self->_normalize_name($col));
-        ($remote_relname) = $self->_inflect_singular($col);
+        ($local_relname) = $self->_inflect_singular($col);
     }
     else {
-        ($remote_relname) = $self->_inflect_singular($self->_normalize_name($remote_table));
+        ($local_relname) = $self->_inflect_singular($self->_normalize_name($remote_table));
     }
 
-    return $remote_relname;
+    return $local_relname;
 }
 
 sub _resolve_relname_collision {
@@ -363,7 +379,7 @@ EOF
 
 sub generate_code {
     my ($self, $tables) = @_;
-    
+
     # make a copy to destroy
     my @tables = @$tables;
 
@@ -394,34 +410,55 @@ sub generate_code {
             }
 
             my %cond;
-            foreach my $i (0 .. $#$local_cols) {
-                $cond{$remote_cols->[$i]} = $local_cols->[$i];
-            }
+            @cond{@$remote_cols} = @$local_cols;
 
             my ( $local_relname, $remote_relname, $remote_method ) =
                 $self->_relnames_and_method( $local_moniker, $rel, \%cond,  $uniqs, \%counters );
             my $local_method  = 'belongs_to';
 
-            ($remote_relname) = $self->_rel_name_map($remote_relname, $local_method, $local_class, $local_moniker, $local_cols, $remote_class, $remote_moniker, $remote_cols);
-            ($local_relname)  = $self->_rel_name_map($local_relname, $remote_method, $remote_class, $remote_moniker, $remote_cols, $local_class, $local_moniker, $local_cols);
-
-            $remote_relname   = $self->_resolve_relname_collision($local_moniker,  $local_cols,  $remote_relname);
-            $local_relname    = $self->_resolve_relname_collision($remote_moniker, $remote_cols, $local_relname);
-
-            push(@{$all_code->{$local_class}},
-                { method => $local_method,
-                  args => [ $remote_relname,
-                            $remote_class,
-                            \%cond,
-                            $self->_remote_attrs($local_moniker, $local_cols),
-                  ],
-                  extra => {
-                      local_class    => $local_class,
-                      local_moniker  => $local_moniker,
-                      remote_moniker => $remote_moniker,
-                  },
-                }
+            ($local_relname) = $self->_rel_name_map(
+                $local_relname, $local_method,
+                $local_class, $local_moniker, $local_cols,
+                $remote_class, $remote_moniker, $remote_cols,
             );
+            ($remote_relname) = $self->_rel_name_map(
+                $remote_relname, $remote_method,
+                $remote_class, $remote_moniker, $remote_cols,
+                $local_class, $local_moniker, $local_cols,
+            );
+
+            $local_relname = $self->_resolve_relname_collision(
+                $local_moniker, $local_cols, $local_relname,
+            );
+            $remote_relname = $self->_resolve_relname_collision(
+                $remote_moniker, $remote_cols, $remote_relname,
+            );
+
+            my $rel_attrs_params = {
+                rel_name      => $local_relname,
+                rel_type      => $local_method,
+                local_source  => $self->schema->source($local_moniker),
+                remote_source => $self->schema->source($remote_moniker),
+                local_table   => $rel->{local_table},
+                local_cols    => $local_cols,
+                remote_table  => $rel->{remote_table},
+                remote_cols   => $remote_cols,
+            };
+
+            push @{$all_code->{$local_class}}, {
+                method => $local_method,
+                args => [
+                    $local_relname,
+                    $remote_class,
+                    \%cond,
+                    $self->_remote_attrs($local_moniker, $local_cols, $rel->{attrs}, $rel_attrs_params),
+                ],
+                extra => {
+                    local_class    => $local_class,
+                    local_moniker  => $local_moniker,
+                    remote_moniker => $remote_moniker,
+                },
+            };
 
             my %rev_cond = reverse %cond;
             for (keys %rev_cond) {
@@ -429,20 +466,31 @@ sub generate_code {
                 delete $rev_cond{$_};
             }
 
-            push(@{$all_code->{$remote_class}},
-                { method => $remote_method,
-                  args => [ $local_relname,
-                            $local_class,
-                            \%rev_cond,
-                            $self->_relationship_attrs($remote_method),
-                  ],
-                  extra => {
-                      local_class    => $remote_class,
-                      local_moniker  => $remote_moniker,
-                      remote_moniker => $local_moniker,
-                  },
-                }
-            );
+            $rel_attrs_params = {
+                rel_name      => $remote_relname,
+                rel_type      => $remote_method,
+                local_source  => $self->schema->source($remote_moniker),
+                remote_source => $self->schema->source($local_moniker),
+                local_table   => $rel->{remote_table},
+                local_cols    => $remote_cols,
+                remote_table  => $rel->{local_table},
+                remote_cols   => $local_cols,
+            };
+
+            push @{$all_code->{$remote_class}}, {
+                method => $remote_method,
+                args => [
+                    $remote_relname,
+                    $local_class,
+                    \%rev_cond,
+                    $self->_relationship_attrs($remote_method, {}, $rel_attrs_params),
+                ],
+                extra => {
+                    local_class    => $remote_class,
+                    local_moniker  => $remote_moniker,
+                    remote_moniker => $local_moniker,
+                },
+            };
         }
     }
 
@@ -464,121 +512,105 @@ sub generate_code {
 sub _generate_m2ms {
     my ($self, $all_code) = @_;
 
-    while (my ($class, $rels) = each %$all_code) {
-        next unless (grep $_->{method} eq 'belongs_to', @$rels) == 2;
+    LINK_CLASS:
+    foreach my $link_class (sort keys %$all_code) {
+        my @rels = grep $_->{method} eq 'belongs_to', @{$all_code->{$link_class}};
+        next unless @rels == 2;
 
-        my $class1_local_moniker  = $rels->[0]{extra}{remote_moniker};
-        my $class1_remote_moniker = $rels->[1]{extra}{remote_moniker};
+        my @class;
+        foreach my $this (0, 1) {
+            my $that = $this ? 0 : 1;
+            my %class;
+            $class[$this] = \%class;
+            $class{local_moniker}  = $rels[$this]{extra}{remote_moniker};
+            $class{remote_moniker} = $rels[$that]{extra}{remote_moniker};
 
-        my $class2_local_moniker  = $rels->[1]{extra}{remote_moniker};
-        my $class2_remote_moniker = $rels->[0]{extra}{remote_moniker};
+            $class{class} = $rels[$this]{args}[1];
 
-        my $class1 = $rels->[0]{args}[1];
-        my $class2 = $rels->[1]{args}[1];
+            my %link_cols = map { $_ => 1 } apply { s/^self\.//i } values %{ $rels[$this]{args}[2] };
 
-        my $class1_to_link_table_rel = first {
-            $_->{method} eq 'has_many' && $_->{args}[1] eq $class
-        } @{ $all_code->{$class1} };
+            $class{link_table_rel} = first {
+                $_->{method} eq 'has_many'
+                    and
+                $_->{args}[1] eq $link_class
+                    and
+                all { $link_cols{$_} } apply { s/^foreign\.//i } keys %{$_->{args}[2]}
+            } @{ $all_code->{$class{class}} };
 
-        my $class1_to_link_table_rel_name = $class1_to_link_table_rel->{args}[0];
+            next LINK_CLASS unless $class{link_table_rel};
 
-        my $class2_to_link_table_rel = first {
-            $_->{method} eq 'has_many' && $_->{args}[1] eq $class
-        } @{ $all_code->{$class2} };
+            $class{link_table_rel_name} = $class{link_table_rel}{args}[0];
 
-        my $class2_to_link_table_rel_name = $class2_to_link_table_rel->{args}[0];
+            $class{link_rel} = $rels[$that]{args}[0];
 
-        my $class1_link_rel = $rels->[1]{args}[0];
-        my $class2_link_rel = $rels->[0]{args}[0];
+            $class{from_cols} = [ apply { s/^self\.//i } values %{
+                $class{link_table_rel}->{args}[2]
+            } ];
 
-        my @class1_from_cols = apply { s/^self\.//i } values %{
-            $class1_to_link_table_rel->{args}[2]
-        };
+            $class{to_cols} = [ apply { s/^foreign\.//i } keys %{ $rels[$that]{args}[2] } ];
 
-        my @class1_link_cols = apply { s/^self\.//i } values %{ $rels->[1]{args}[2] };
+            $class{from_link_cols} = [ apply { s/^self\.//i } values %{ $rels[$this]{args}[2] } ];
+        }
 
-        my @class1_to_cols = apply { s/^foreign\.//i } keys %{ $rels->[1]{args}[2] };
-
-        my @class2_from_cols = apply { s/^self\.//i } values %{
-            $class2_to_link_table_rel->{args}[2]
-        };
-
-        my @class2_link_cols = apply { s/^self\.//i } values %{ $rels->[0]{args}[2] };
-
-        my @class2_to_cols = apply { s/^foreign\.//i } keys %{ $rels->[0]{args}[2] };
+        my $link_moniker = $rels[0]{extra}{local_moniker};
 
         my @link_table_cols =
-            @{[ $self->schema->source($rels->[0]{extra}{local_moniker})->columns ]};
+            @{[ $self->schema->source($link_moniker)->columns ]};
 
         my @link_table_primary_cols =
-            @{[ $self->schema->source($rels->[0]{extra}{local_moniker})->primary_columns ]};
+            @{[ $self->schema->source($link_moniker)->primary_columns ]};
 
-        next unless @class1_link_cols + @class2_link_cols == @link_table_cols
-            && @link_table_cols == @link_table_primary_cols;
+        next unless array_eq(
+            [ sort +uniq @{$class[0]{from_link_cols}}, @{$class[1]{from_link_cols}} ],
+            [ sort @link_table_primary_cols ],
+        ) && ($self->allow_extra_m2m_cols || @link_table_cols == @link_table_primary_cols);
 
-        my ($class1_to_class2_relname) = $self->_rel_name_map(
-            ($self->_inflect_plural($class1_link_rel))[0],
-            'many_to_many',
-            $class1,
-            $class1_local_moniker,
-            \@class1_from_cols,
-            $class2,
-            $class1_remote_moniker,
-            \@class1_to_cols,
-        );
+        foreach my $this (0, 1) {
+            my $that = $this ? 0 : 1;
+            ($class[$this]{m2m_relname}) = $self->_rel_name_map(
+                ($self->_inflect_plural($class[$this]{link_rel}))[0],
+                'many_to_many',
+                @{$class[$this]}{qw(class local_moniker from_cols)},
+                $class[$that]{class},
+                @{$class[$this]}{qw(remote_moniker to_cols)},
+                {
+                    link_class => $link_class,
+                    link_moniker => $link_moniker,
+                    link_rel_name => $class[$this]{link_table_rel_name},
+                },
+            );
 
-        $class1_to_class2_relname = $self->_resolve_relname_collision(
-            $class1_local_moniker,
-            \@class1_from_cols,
-            $class1_to_class2_relname,
-        );
+            $class[$this]{m2m_relname} = $self->_resolve_relname_collision(
+                @{$class[$this]}{qw(local_moniker from_cols m2m_relname)},
+            );
+        }
 
-        my ($class2_to_class1_relname) = $self->_rel_name_map(
-            ($self->_inflect_plural($class2_link_rel))[0],
-            'many_to_many',
-            $class1,
-            $class2_local_moniker,
-            \@class2_from_cols,
-            $class2,
-            $class2_remote_moniker,
-            \@class2_to_cols,
-        );
+        for my $this (0, 1) {
+            my $that = $this ? 0 : 1;
 
-        $class2_to_class1_relname = $self->_resolve_relname_collision(
-            $class2_local_moniker,
-            \@class2_from_cols,
-            $class2_to_class1_relname,
-        );
-
-        push @{$all_code->{$class1}}, {
-            method => 'many_to_many',
-            args   => [
-                $class1_to_class2_relname,
-                $class1_to_link_table_rel_name,
-                $class1_link_rel,
-            ],
-            extra  => {
-                local_class    => $class1,
-                link_class     => $class,
-                local_moniker  => $class1_local_moniker,
-                remote_moniker => $class1_remote_moniker,
-            },
-        };
-
-        push @{$all_code->{$class2}}, {
-            method => 'many_to_many',
-            args   => [
-                $class2_to_class1_relname,
-                $class2_to_link_table_rel_name,
-                $class2_link_rel,
-            ],
-            extra  => {
-                local_class    => $class2,
-                link_class     => $class,
-                local_moniker  => $class2_local_moniker,
-                remote_moniker => $class2_remote_moniker,
-            },
-        };
+            push @{$all_code->{$class[$this]{class}}}, {
+                method => 'many_to_many',
+                args   => [
+                    @{$class[$this]}{qw(m2m_relname link_table_rel_name link_rel)},
+                    $self->_relationship_attrs('many_to_many', {}, {
+                        rel_type => 'many_to_many',
+                        rel_name => $class[$this]{class2_relname},
+                        local_source => $self->schema->source($class[$this]{local_moniker}),
+                        remote_source => $self->schema->source($class[$this]{remote_moniker}),
+                        local_table => $self->loader->class_to_table->{$class[$this]{class}},
+                        local_cols => $class[$this]{from_cols},
+                        remote_table => $self->loader->class_to_table->{$class[$that]{class}},
+                        remote_cols => $class[$that]{from_cols},
+                    }) || (),
+                ],
+                extra  => {
+                    local_class    => $class[$this]{class},
+                    link_class     => $link_class,
+                    local_moniker  => $class[$this]{local_moniker},
+                    remote_moniker => $class[$this]{remote_moniker},
+                },
+            };
+        }
     }
 }
 
@@ -779,7 +811,7 @@ sub _disambiguate {
                 my $rel_name_mapped;
 
                 ($relname_new, $rel_name_mapped) = $self->_rel_name_map($relname_new, $rel->{method}, $local_class, $local_moniker, \@from_cols, $to_class, $remote_moniker, \@to_cols);
-                
+
                 my $mapped = $inflect_mapped || $rel_name_mapped;
 
                 warn <<"EOF" unless $mapped;
@@ -803,15 +835,15 @@ sub _relnames_and_method {
     my $remote_moniker  = $rel->{remote_source};
     my $remote_obj      = $self->schema->source( $remote_moniker );
     my $remote_class    = $self->schema->class(  $remote_moniker );
-    my $remote_relname  = $self->_remote_relname( $rel->{remote_table}, $cond);
+    my $local_relname   = $self->_local_relname( $rel->{remote_table}, $cond);
 
     my $local_cols      = $rel->{local_columns};
     my $local_table     = $rel->{local_table};
     my $local_class     = $self->schema->class($local_moniker);
     my $local_source    = $self->schema->source($local_moniker);
 
-    my $local_relname_uninflected = $self->_normalize_name($local_table);
-    my ($local_relname) = $self->_inflect_plural($self->_normalize_name($local_table));
+    my $remote_relname_uninflected = $self->_normalize_name($local_table);
+    my ($remote_relname) = $self->_inflect_plural($self->_normalize_name($local_table));
 
     my $remote_method = 'has_many';
 
@@ -819,7 +851,7 @@ sub _relnames_and_method {
     if (array_eq([ $local_source->primary_columns ], $local_cols) ||
             first { array_eq($_->[1], $local_cols) } @$uniqs) {
         $remote_method   = 'might_have';
-        ($local_relname) = $self->_inflect_singular($local_relname_uninflected);
+        ($remote_relname) = $self->_inflect_singular($remote_relname_uninflected);
     }
 
     # If more than one rel between this pair of tables, use the local
@@ -843,9 +875,9 @@ sub _relnames_and_method {
                 push @{ $self->_temp_classes }, $class;
             }
 
-            if ($class->has_relationship($local_relname)) {
+            if ($class->has_relationship($remote_relname)) {
                 my $rel_cols = [ sort { $a cmp $b } apply { s/^foreign\.//i }
-                    (keys %{ $class->relationship_info($local_relname)->{cond} }) ];
+                    (keys %{ $class->relationship_info($remote_relname)->{cond} }) ];
 
                 $relationship_exists = 1 if array_eq([ sort @$local_cols ], $rel_cols);
             }
@@ -853,16 +885,16 @@ sub _relnames_and_method {
 
         if (not $relationship_exists) {
             my $colnames = q{_} . $self->_normalize_name(join '_', @$local_cols);
-            $remote_relname .= $colnames if keys %$cond > 1;
+            $local_relname .= $colnames if keys %$cond > 1;
 
-            $local_relname = $self->_strip_id_postfix($self->_normalize_name($local_table . $colnames));
+            $remote_relname = $self->_strip_id_postfix($self->_normalize_name($local_table . $colnames));
 
-            $local_relname_uninflected = $local_relname;
-            ($local_relname) = $self->_inflect_plural($local_relname);
+            $remote_relname_uninflected = $remote_relname;
+            ($remote_relname) = $self->_inflect_plural($remote_relname);
 
             # if colnames were added and this is a might_have, re-inflect
             if ($remote_method eq 'might_have') {
-                ($local_relname) = $self->_inflect_singular($local_relname_uninflected);
+                ($remote_relname) = $self->_inflect_singular($remote_relname_uninflected);
             }
         }
     }
@@ -872,9 +904,10 @@ sub _relnames_and_method {
 
 sub _rel_name_map {
     my ($self, $relname, $method, $local_class, $local_moniker, $local_cols,
-        $remote_class, $remote_moniker, $remote_cols) = @_;
+        $remote_class, $remote_moniker, $remote_cols, $extra) = @_;
 
     my $info = {
+        %{$extra || {}},
         name           => $relname,
         type           => $method,
         local_class    => $local_class,
@@ -885,9 +918,13 @@ sub _rel_name_map {
         remote_columns => $remote_cols,
     };
 
-    my $new_name = $relname;
+    $self->_run_user_map($self->rel_name_map, $info);
+}
 
-    my $map = $self->rel_name_map;
+sub _run_user_map {
+    my ($self, $map, $info) = @_;
+
+    my $new_name = $info->{name};
     my $mapped = 0;
 
     if ('HASH' eq ref($map)) {
@@ -905,7 +942,14 @@ sub _rel_name_map {
         }
     }
     elsif ('CODE' eq ref($map)) {
-        my $name = $map->($info);
+        my $cb = sub {
+            my ($cb_map) = @_;
+            croak "reentered rel_name_map must be a hashref"
+                unless 'HASH' eq ref($cb_map);
+            my ($cb_name, $cb_mapped) = $self->_run_user_map($cb_map, $info);
+            return $cb_mapped && $cb_name;
+        };
+        my $name = $map->($info, $cb);
         if ($name) {
             $new_name = $name;
             $mapped   = 1;
@@ -925,9 +969,9 @@ sub _cleanup {
     $self->_temp_classes([]);
 }
 
-=head1 AUTHOR
+=head1 AUTHORS
 
-See L<DBIx::Class::Schema::Loader/AUTHOR> and L<DBIx::Class::Schema::Loader/CONTRIBUTORS>.
+See L<DBIx::Class::Schema::Loader/AUTHORS>.
 
 =head1 LICENSE
 
